# $Id: shared.sh.in,v 1.58 2005/03/01 21:05:11 cph Exp $
#
# Copyright 2000,2001,2002,2003,2005 Massachusetts Institute of Technology
#
# This file is part of laptop-net.
#
# Laptop-net is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Laptop-net is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with laptop-net; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Shared code for network-interface control scripts.

STATE_DIR="/var/run/laptop-net"
RESUME_FILE="${STATE_DIR}/resume"
PROFILE_FILE="${STATE_DIR}/profile"
IP_MAP_FILE="/etc/laptop-net/ip-map"
SCHEME_CONFIG_FILE="/etc/laptop-net/schemes"
CONFIG_FILE="/etc/default/laptop-net"
RESOLV_CONF="/etc/resolv.conf"
SCHEME_DIR="/var/lib/laptop-net/schemes"
PROFILE_CHANGE="/usr/share/laptop-net/profile-change"
ARP_DISCOVERY="/usr/lib/laptop-net/arp-discovery"
UPGRADING_FILE="/usr/share/laptop-net/.upgrading"

RESOLVCONF_PROGRAM="/sbin/resolvconf"
LOGGER_PROGRAM="/usr/bin/logger"
if [ -x "${LOGGER_PROGRAM}" ]; then
    LOGGER="${LOGGER_PROGRAM} -p daemon.info -t laptop-net"
else
    LOGGER="true"
fi

LINKUP="/usr/lib/laptop-net/linkup"
MKTONE="/usr/lib/laptop-net/mktone 150"
MKTONE_UP=1000
MKTONE_DOWN=2000
MKTONE_ERROR=4000

LINUX_VERSION=$(uname --release | awk -F. '{ print $1 }')
LINUX_PATCHLEVEL=$(uname --release | awk -F. '{ print $2 }')
LINUX_SUBLEVEL=$(uname --release | awk -F. '
{
    where = match ($3, "[0-9]+")
    if (where != 0)
	print substr ($3, RSTART, RLENGTH)
}')

kernel_at_least_2_4_p ()
(
    if [ "${LINUX_VERSION}" -lt 2 ]; then
	return 1
    elif [ "${LINUX_VERSION}" -eq 2 ] && [ "${LINUX_PATCHLEVEL}" -lt 4 ]; then
	return 1
    else
	return 0
    fi
)

kernel_supports_new_mii_p ()
(
    if [ "${LINUX_VERSION}" -lt 2 ]; then
	return 1
    elif [ "${LINUX_VERSION}" -gt 2 ]; then
	return 0
    elif [ "${LINUX_PATCHLEVEL}" -lt 4 ]; then
	return 1
    elif [ "${LINUX_PATCHLEVEL}" -gt 4 ]; then
	return 0
    elif [ "${LINUX_SUBLEVEL}" -lt 18 ]; then
	return 1
    else
	return 0
    fi
)

MODULE_NAME=
MII_SUPPORTED=
[ ! -r "${CONFIG_FILE}" ] || . "${CONFIG_FILE}"

if [ "${MODULE_NAME}" = "none" ]; then
    MODULE_NAME=
fi

if [ -z "${MII_SUPPORTED}" ]; then
    if kernel_supports_new_mii_p || [ -z "${MODULE_NAME}" ]; then
	MII_SUPPORTED="yes"
    elif kernel_at_least_2_4_p; then
	case "${MODULE_NAME}" in
	(3c59x | 8139too | au1000_eth | eepro100 | epic100 | fealnx \
	| hamachi | ioc3-eth | natsemi | pcnet32 | sis900 | starfire \
	| sundance | sungem | tlan | tulip | via-rhine | winbond-840 \
	| yellowfin)
	    MII_SUPPORTED="yes"
	    ;;
	(*)
	    MII_SUPPORTED="no"
	    ;;
	esac
    else
	# 2.2.x kernels
	case "${MODULE_NAME}" in
	(3c59x | eepro100 | epic100 | old_tulip | pcnet32 | rtl8139 \
	| sis900 | tulip | via-rhine | yellowfin)
	    MII_SUPPORTED="yes"
	    ;;
	(*)
	    MII_SUPPORTED="no"
	    ;;
	esac
    fi
fi

# Disable use of LINKUP if MII not supported.
[ "${MII_SUPPORTED}" = "yes" ] || LINKUP="/bin/true"

state_dir_name ()
(
    INTERFACE="${1}"
    DIR="${STATE_DIR}/${INTERFACE}"
    [ -d "${DIR}" ] || mkdir "${DIR}"
    echo "${DIR}"
)

doing_upgrade ()
(
    FILE="${UPGRADING_FILE}"
    if test -f "${FILE}"; then
	current=`cat "${FILE}"`
    else
	current="unknown"
    fi
    if [ "${current}" = "yes" ] ; then
       return 0
    else
       return 1
    fi
)

state_file_name ()
(
    echo "$(state_dir_name "${1}")/state"
)

read_state_file ()
(
    FILE="$(state_file_name "${1}")"
    if test -f "${FILE}"; then
	cat "${FILE}"
    else
	echo "unknown"
    fi
)

write_state_file ()
(
    echo "${2}" > "$(state_file_name "${1}")"
)

write_target_file ()
(
    INTERFACE="${1}"
    TARGET="${2}"
    FILE="$(state_dir_name "${INTERFACE}")/target"
    if ! lockfile-create --retry 4 "${FILE}"; then
	${LOGGER} "Unable to lock \"${FILE}\"."
	return 1
    fi
    if ! test -f "${FILE}"; then
	echo "${TARGET}" > "${FILE}"
	lockfile-remove "${FILE}"
	return 0
    fi
    OTARGET="$(cat "${FILE}")"
    OSTATE="$(read_state_file "${INTERFACE}")"
    if test "${OTARGET}" != "${TARGET}" -a "${OTARGET}" = "${OSTATE}"; then
	echo "${TARGET}" > "${FILE}"
	lockfile-remove "${FILE}"
	return 0
    else
	lockfile-remove "${FILE}"
	return 1
    fi
)

remove_target_file ()
(
    INTERFACE="${1}"
    FILE="$(state_dir_name "${INTERFACE}")/target"
    if ! lockfile-create --retry 4 "${FILE}"; then
	${LOGGER} "Unable to lock \"${FILE}\"."
	return 0
    fi
    rm -f "${FILE}" || true
    lockfile-remove "${FILE}"
    return 0
)

clean_state ()
(
    rm -f "${STATE_DIR}"/*/*
)

scheme_file_name ()
(
    echo "$(state_dir_name "${1}")/scheme"
)

scheme_source_file_name ()
(
    echo "$(state_dir_name "${1}")/scheme-source"
)

read_scheme_file ()
(
    FILE="$(scheme_file_name "${1}")"
    [ -f "${FILE}" ] || echo "down" > "${FILE}"
    cat "${FILE}"
)

read_scheme_source ()
(
    INTERFACE="${1}"
    FILE="$(scheme_source_file_name "${INTERFACE}")"
    if [ -f "${FILE}" ]; then
	cat "${FILE}"
    else
	echo "unknown"
    fi
)

choose_scheme ()
(
    INTERFACE="${1}"
    TEMP="$(state_dir_name "${1}")/scheme.tmp"
    rm -f "${TEMP}"
    if "${ARP_DISCOVERY}" "${INTERFACE}" "${IP_MAP_FILE}" > "${TEMP}"
    then
	NEW_SCHEME="$(cat "${TEMP}")"
	[ "${NEW_SCHEME}" ] || NEW_SCHEME="@NO-CHOICES@"
    else
	NEW_SCHEME="@NO-CHOICES@"
    fi
    case "${NEW_SCHEME}" in
    ("@NO-CHOICES@")
	${LOGGER} "No change to network scheme"
	;;
    ("@NO-RESPONSES@")
	if [ "$(read_scheme_source "${INTERFACE}")" = "arp-discovery" ]; then
	    write_scheme "${INTERFACE}" "default" "arp-discovery"
	else
	    ${LOGGER} "No change to network scheme"
	fi
	;;
    (*)
	write_scheme "${INTERFACE}" "${NEW_SCHEME}" "arp-discovery"
	;;
    esac
    rm -f "${TEMP}"
)

read_scheme_config ()
{
    INTERFACE="${1}"
    SCHEME="$(read_scheme_file "${INTERFACE}")"
    DHCP=
    ADDRESS=
    NETMASK=
    BROADCAST=
    NETWORK=
    GATEWAY=
    DOMAIN=
    SEARCH=
    NAMESERVERS=
    START_CMD=
    STOP_CMD=
    if [ -e "${SCHEME_CONFIG_FILE}" ]; then
	. "${SCHEME_CONFIG_FILE}"
    else
	[ "${SCHEME}" = "offline" ] || DHCP="yes"
    fi
    export LTN_INTERFACE="${INTERFACE}"
    export LTN_SCHEME="${SCHEME}"
    export LTN_DHCP="${DHCP}"
    export LTN_ADDRESS="${ADDRESS}"
    export LTN_NETMASK="${NETMASK}"
    export LTN_BROADCAST="${BROADCAST}"
    export LTN_NETWORK="${NETWORK}"
    export LTN_GATEWAY="${GATEWAY}"
    export LTN_DOMAIN="${DOMAIN}"
    export LTN_SEARCH="${SEARCH}"
    export LTN_NAMESERVERS="${NAMESERVERS}"
    export LTN_START_CMD="${START_CMD}"
    export LTN_STOP_CMD="${STOP_CMD}"
}

write_scheme ()
(
    INTERFACE="${1}"
    NAME="${2}"
    SOURCE="${3}"
    ${LOGGER} "Network scheme \"${NAME}\" selected for \"${INTERFACE}\" by \"${SOURCE}\"."
    echo "${NAME}" > "$(scheme_file_name "${INTERFACE}")"
    echo "${SOURCE}" > "$(scheme_source_file_name "${INTERFACE}")"
)

bringup ()
(
    INTERFACE="${1}"
    WHY="${2}"
    write_target_file "${INTERFACE}" "up" || return 0
    trap "remove_target_file ${INTERFACE}" EXIT HUP TERM
    choose_scheme "${INTERFACE}"
    read_scheme_config "${INTERFACE}"
    IFACES_FILE="${SCHEME_DIR}/${SCHEME}"
    if write_interfaces_file "${INTERFACE}" "${IFACES_FILE}"; then
	${LOGGER} "Starting network interface \"${INTERFACE}\""
	ifup --interfaces="${IFACES_FILE}" --force "${INTERFACE}"
	write_resolv_conf "${INTERFACE}"
	"${PROFILE_CHANGE}" "${INTERFACE}"
	if "${LINKUP}" "${INTERFACE}"; then
	    ${MKTONE} "${MKTONE_UP}"
	else
	    ${MKTONE} "${MKTONE_ERROR}"
	fi
	write_state_file "${INTERFACE}" "up"
	trap - EXIT HUP TERM
    fi
)

bringdown ()
(
    if doing_upgrade ; then
       return 0
    fi
    INTERFACE="${1}"
    WHY="${2}"
    write_target_file "${INTERFACE}" "down" || return 0
    trap "remove_target_file ${INTERFACE}" EXIT HUP TERM
    read_scheme_config "${INTERFACE}"
    IFACES_FILE="${SCHEME_DIR}/${SCHEME}"
    if write_interfaces_file "${INTERFACE}" "${IFACES_FILE}"; then
	# Don't try this if the interface is already down.
	if "${LINKUP}" "${INTERFACE}"; then
	    shut_down_nfs "${INTERFACE}"
	fi
	${LOGGER} "Stopping network interface \"${INTERFACE}\""
	ifdown --interfaces="${IFACES_FILE}" --force "${INTERFACE}"
	"${PROFILE_CHANGE}" "${INTERFACE}"
	${MKTONE} "${MKTONE_DOWN}"
	write_state_file "${INTERFACE}" "down"
	trap - EXIT HUP TERM
    fi
)

shut_down_nfs ()
(
    INTERFACE="${1}"
    # Shut down all NFS mounts on this interface.
    # (Shamelessly appropriated from pcmcia-cs.)
    [ -x /sbin/ifuser ] || return 0
    [ -x /bin/fuser ] || return 0
    if doing_upgrade ; then
        return 0
    else
        echo -n "Unmounting remote filesystems"
        mount -t nfs | sed -e 's/:.* on \(.*\) type .*/ \1/' | shut_down_nfs_1
        echo "."
    fi
)

shut_down_nfs_1 ()
(
    if read HOST MT ; then
	shut_down_nfs_1
	case "${HOST}" in
	(localhost | 127.0.0.1)
	    ;;
	(*)
	    if ifuser "${INTERFACE}" "${HOST}" ; then
		fuser -k -m "${MT}" > /dev/null || true
		umount -f -v "${MT}"
	    fi
	    ;;
	esac
    fi
)

load_module ()
(
    INTERFACE="${1}"
    if [ "${MODULE_NAME}" ]; then
	modprobe -q "${MODULE_NAME}" || true
    fi
    return 0
)

unload_module ()
(
    INTERFACE="${1}"
    if [ "${MODULE_NAME}" ]; then
	ifconfig "${INTERFACE}" down 2> /dev/null || true
	modprobe -r -q "${MODULE_NAME}" || true
    fi
    return 0
)

maybe_restart_dhcp ()
(
    if [ -f "/etc/debian_version" ] && [ "${DHCP}" = "yes" ]; then
	for PKG in dhcp-client pump dhcpcd; do
	    if [ -x "/etc/init.d/${PKG}" ]; then
		"/etc/init.d/${PKG}" restart
		break
	    fi
	done
    fi
)

kernel_supports_power_management ()
(
    # For now, don't take advantage of 2.4.x suspend support.
    return 1
)

write_interfaces_file ()
(
    INTERFACE="${1}"
    FILENAME="${2}"
    NEWFILE="${FILENAME}.new"
    rm -f "${NEWFILE}"
    if [ "${LTN_DHCP}" = "yes" ]; then
	echo "iface ${INTERFACE} inet dhcp" > "${NEWFILE}"
	# The dns_ entries are handled by resolvconf (if installed).
	if [ "${LTN_DOMAIN}" ]; then
	    echo "    dns_domain ${LTN_DOMAIN}" >> "${NEWFILE}"
	fi
	if [ "${LTN_SEARCH}" ]; then
	    echo "    dns_search ${LTN_SEARCH}" >> "${NEWFILE}"
	fi
	if [ "${LTN_NAMESERVERS}" ]; then
	    echo "    dns_nameservers ${LTN_NAMESERVERS}" >> "${NEWFILE}"
	fi
	WRITE_FILE="yes"
    elif [ "${LTN_ADDRESS}" ] && [ "${LTN_NETMASK}" ]; then
	echo "iface ${INTERFACE} inet static" > "${NEWFILE}"
	echo "    address ${LTN_ADDRESS}" >> "${NEWFILE}"
	echo "    netmask ${LTN_NETMASK}" >> "${NEWFILE}"
	if [ "${LTN_BROADCAST}" ]; then
	    echo "    broadcast ${LTN_BROADCAST}" >> "${NEWFILE}"
	fi
	if [ "${LTN_NETWORK}" ]; then
	    echo "    network ${LTN_NETWORK}" >> "${NEWFILE}"
	fi
	if [ "${LTN_GATEWAY}" ]; then
	    echo "    gateway ${LTN_GATEWAY}" >> "${NEWFILE}"
	fi
	# The dns_ entries are handled by resolvconf (if installed).
	if [ "${LTN_DOMAIN}" ]; then
	    echo "    dns_domain ${LTN_DOMAIN}" >> "${NEWFILE}"
	fi
	if [ "${LTN_SEARCH}" ]; then
	    echo "    dns_search ${LTN_SEARCH}" >> "${NEWFILE}"
	fi
	if [ "${LTN_NAMESERVERS}" ]; then
	    echo "    dns_nameservers ${LTN_NAMESERVERS}" >> "${NEWFILE}"
	fi
	WRITE_FILE="yes"
    else
	WRITE_FILE="no"
    fi

    if [ "${WRITE_FILE}" = "yes" ]; then
	if [ "${LTN_START_CMD}" ]; then
	    echo "    up ${LTN_START_CMD}" >> "${NEWFILE}"
	fi
	if [ "${LTN_STOP_CMD}" ]; then
	    echo "    down ${LTN_STOP_CMD}" >> "${NEWFILE}"
	fi
	chmod 644 "${NEWFILE}"
	rm -f "${FILENAME}"
	mv -f "${NEWFILE}" "${FILENAME}"
    fi

    [ "${WRITE_FILE}" = "yes" ]
)

write_resolv_conf ()
(
    INTERFACE="${1}"
    # If using resolvconf, this is handled by ifup/ifdown.
    [ -x "${RESOLVCONF_PROGRAM}" ] && return 0
    # If using DHCP, this is handled by DHCP client.
    [ "${LTN_DHCP}" = "yes" ] && return 0
    [ "${RESOLV_CONF}" ] || return 0
    # Create a temporary file with the new resolver info.
    RESOLV_TMP="${RESOLV_CONF}.tmp"
    echo "# ${INTERFACE} begin" > "${RESOLV_TMP}"
    [ "${LTN_DOMAIN}" ] && echo "domain ${LTN_DOMAIN}" >> "${RESOLV_TMP}"
    [ "${LTN_SEARCH}" ] && echo "search ${LTN_SEARCH}" >> "${RESOLV_TMP}"
    for NAMESERVER in ${LTN_NAMESERVERS}; do
	echo "nameserver ${NAMESERVER}" >> "${RESOLV_TMP}"
    done
    echo "# ${INTERFACE} end" >> "${RESOLV_TMP}"
    # Merge the new info into the existing resolv.conf file.
    if [ -f "${RESOLV_CONF}" ]; then
	# Omit previous entries for this interface.
	SED_SCRIPT="/# ${INTERFACE} begin/,/# ${INTERFACE} end/d"
	# Omit other related entries.
	[ "${LTN_SEARCH}" ] && SED_SCRIPT="${SED_SCRIPT}; /^search/d"
	[ "${LTN_NAMESERVERS}" ] && SED_SCRIPT="${SED_SCRIPT}; /^nameserver/d"
	[ "${LTN_DOMAIN}" ] && SED_SCRIPT="${SED_SCRIPT}; /^domain/d"
	# Omit previously commented-out entries.
	SED_SCRIPT="${SED_SCRIPT}; /^#search/d"
	SED_SCRIPT="${SED_SCRIPT}; /^#nameserver/d"
	SED_SCRIPT="${SED_SCRIPT}; /^#domain/d"
	sed -e "${SED_SCRIPT}" "${RESOLV_CONF}" >> "${RESOLV_TMP}"
	# Do nothing if no changes. 
	cmp --quiet "${RESOLV_TMP}" "${RESOLV_CONF}" && return 0
	# Back up current file if possible.
	cp -fp "${RESOLV_CONF}" "${RESOLV_CONF}.old"
    fi
    # Install new file.
    mv -f "${RESOLV_TMP}" "${RESOLV_CONF}"
)
